@@ -1,7 +1,7 @@
 /*
  * dhcp.c	Functions to send/receive dhcp packets.
  *
- * Version:	$Id: dhcp.c,v 1.1 2008/04/20 14:51:25 aland Exp $
+ * Version:	$Id: dhcp.c,v 1.2 2008/05/28 17:32:26 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: dhcp.c,v 1.1 2008/04/20 14:51:25 aland Exp $")
+RCSID("$Id: dhcp.c,v 1.2 2008/05/28 17:32:26 aland Exp $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/udpfromto.h>
@@ -404,8 +404,8 @@ int fr_dhcp_decode(RADIUS_PACKET *packet)
 			break;
 			
 		case PW_TYPE_ETHERNET:
-			memcpy(vp->vp_ether, p, packet->data[2]);
-			vp->length = packet->data[2];
+			memcpy(vp->vp_ether, p, sizeof(vp->vp_ether));
+			vp->length = sizeof(vp->vp_ether);
 			break;
 			
 		default:
@@ -949,6 +949,11 @@ int fr_dhcp_encode(RADIUS_PACKET *packet, RADIUS_PACKET *original)
 				vp->length = packet->data[2];
 				break;
 				
+			case PW_TYPE_ETHERNET: /* only for Client HW Address */
+				memcpy(vp->vp_ether, p, sizeof(vp->vp_ether));
+				vp->length = sizeof(vp->vp_ether);
+				break;
+				
 			default:
 				fprintf(stderr, "Internal sanity check failed %d %d\n", vp->type, __LINE__);
 				pairfree(&vp);
