@@ -1,7 +1,7 @@
 /*
  * detail.c	Process the detail file
  *
- * Version:	$Id: detail.c,v 1.8 2008/02/04 21:37:15 aland Exp $
+ * Version:	$Id: detail.c,v 1.16 2008/06/05 09:48:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: detail.c,v 1.8 2008/02/04 21:37:15 aland Exp $")
+RCSID("$Id: detail.c,v 1.16 2008/06/05 09:48:22 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -33,8 +33,14 @@ RCSID("$Id: detail.c,v 1.8 2008/02/04 21:37:15 aland Exp $")
 #include <sys/stat.h>
 #endif
 
+#ifdef HAVE_GLOB_H
+#include <glob.h>
+#endif
+
 #include <fcntl.h>
 
+#ifdef WITH_DETAIL
+
 #define USEC (1000000)
 
 typedef struct listen_detail_t {
@@ -47,6 +53,7 @@ typedef struct listen_detail_t {
 	time_t		timestamp;
 	fr_ipaddr_t	client_ip;
 	int		load_factor; /* 1..100 */
+	int		signal;
 
 	int		has_rtt;
 	int		srtt;
@@ -83,8 +90,10 @@ int detail_send(rad_listen_t *listener, REQUEST *request)
 	 *	caller it's OK to read more "detail" file stuff.
 	 */
 	if (request->reply->code == 0) {
-		radius_signal_self(RADIUS_SIGNAL_SELF_DETAIL);
+		data->delay_time = USEC;
+		data->signal = 1;
 		data->state = STATE_NO_REPLY;
+		radius_signal_self(RADIUS_SIGNAL_SELF_DETAIL);
 		return 0;
 	}
 
@@ -147,17 +156,37 @@ int detail_send(rad_listen_t *listener, REQUEST *request)
 	 *	rtt / (rtt + delay) = load_factor / 100
 	 */
 	data->delay_time = (data->srtt * (100 - data->load_factor)) / (data->load_factor);
+	if (data->delay_time == 0) data->delay_time = USEC / 10;
+
+	/*
+	 *	Cap delay at 4 packets/s.  If the end system can't
+	 *	handle this, then it's very broken.
+	 */
+	if (data->delay_time > (USEC / 4)) data->delay_time= USEC / 4;
 
 #if 0
 	DEBUG2("RTT %d\tdelay %d", data->srtt, data->delay_time);
 #endif
 
 	data->last_packet = now;
+	data->signal = 1;
 	data->state = STATE_REPLIED;
+	radius_signal_self(RADIUS_SIGNAL_SELF_DETAIL);
 
 	return 0;
 }
 
+int detail_delay(rad_listen_t *listener)
+{
+	listen_detail_t *data = listener->data;
+
+	if (!data->signal) return 0;
+
+	data->signal = 0;
+
+	return data->delay_time;
+}
+
 
 /*
  *	Open the detail file, if we can.
@@ -168,13 +197,11 @@ int detail_send(rad_listen_t *listener, REQUEST *request)
 static int detail_open(rad_listen_t *this)
 {
 	struct stat st;
-	char buffer[2048];
 	listen_detail_t *data = this->data;
+	char *filename = data->filename;
 
 	rad_assert(data->state == STATE_UNOPENED);
-	snprintf(buffer, sizeof(buffer), "%s.work", data->filename);
-	free(data->filename_work);
-	data->filename_work = strdup(buffer);
+	data->delay_time = USEC;
 
 	/*
 	 *	Open detail.work first, so we don't lose
@@ -185,10 +212,14 @@ static int detail_open(rad_listen_t *this)
 	 *	we've got to open it for writing in order to
 	 *	establish the lock, to prevent rlm_detail from
 	 *	writing to it.
+	 *
+	 *	This also means that if we're doing globbing,
+	 *	this file will be read && processed before the
+	 *	file globbing is done.
 	 */
-	this->fd = open(buffer, O_RDWR);
+	this->fd = open(data->filename_work, O_RDWR);
 	if (this->fd < 0) {
-		DEBUG2("Polling for detail file %s", data->filename);
+		DEBUG2("Polling for detail file %s", filename);
 
 		/*
 		 *	Try reading the detail file.  If it
@@ -198,40 +229,68 @@ static int detail_open(rad_listen_t *this)
 		 *	exists, even if we don't have permissions
 		 *	to read it.
 		 */
-		if (stat(data->filename, &st) < 0) {
+		if (stat(filename, &st) < 0) {
+#ifdef HAVE_GLOB_H
+			int i, found;
+			time_t ctime;
+			glob_t files;
+
+			memset(&files, 0, sizeof(files));
+			if (glob(filename, 0, NULL, &files) != 0) {
+				return 0;
+			}
+
+			ctime = 0;
+			found = -1;
+			for (i = 0; i < files.gl_pathc; i++) {
+				if (stat(files.gl_pathv[i], &st) < 0) continue;
+
+				if ((i == 0) ||
+				    (st.st_ctime < ctime)) {
+					ctime = st.st_ctime;
+					found = i;
+				}
+			}
+
+			if (found < 0) {
+				globfree(&files);
+				return 0;
+			}
+
+			filename = strdup(files.gl_pathv[found]);
+			globfree(&files);
+#else
 			return 0;
+#endif
 		}
 
 		/*
 		 *	Open it BEFORE we rename it, just to
 		 *	be safe...
 		 */
-		this->fd = open(data->filename, O_RDWR);
+		this->fd = open(filename, O_RDWR);
 		if (this->fd < 0) {
 			radlog(L_ERR, "Failed to open %s: %s",
-			       data->filename, strerror(errno));
+			       filename, strerror(errno));
+			if (filename != data->filename) free(filename);
 			return 0;
 		}
 
 		/*
 		 *	Rename detail to detail.work
 		 */
-		if (rename(data->filename, buffer) < 0) {
+		DEBUG("detail_recv: Renaming %s -> %s", filename, data->filename_work);
+		if (rename(filename, data->filename_work) < 0) {
+			if (filename != data->filename) free(filename);
 			close(this->fd);
 			this->fd = -1;
 			return 0;
 		}
+		if (filename != data->filename) free(filename);
 	} /* else detail.work existed, and we opened it */
 
 	rad_assert(data->vps == NULL);
-
 	rad_assert(data->fp == NULL);
-	data->fp = fdopen(this->fd, "r");
-	if (!data->fp) {
-		radlog(L_ERR, "Failed to re-open %s: %s",
-		       data->filename, strerror(errno));
-		return 0;
-	}
 
 	data->state = STATE_UNLOCKED;
 
@@ -271,11 +330,6 @@ int detail_recv(rad_listen_t *listener,
 	open_file:
 			rad_assert(listener->fd < 0);
 			
-			/*
-			 *	FIXME: If the file doesn't exist, then
-			 *	return "sleep for 1s", to avoid busy
-			 *	looping.
-			 */
 			if (!detail_open(listener)) return 0;
 
 			rad_assert(data->state == STATE_UNLOCKED);
@@ -302,8 +356,24 @@ int detail_recv(rad_listen_t *listener,
 			 *	radrelay.
 			 */
 			if (rad_lockfd_nonblock(listener->fd, 0) < 0) {
+				/*
+				 *	Close the FD.  The main loop
+				 *	will wake up in a second and
+				 *	try again.
+				 */
+				close(listener->fd);
+				listener->fd = -1;
+				data->state = STATE_UNOPENED;
 				return 0;
 			}
+
+			data->fp = fdopen(listener->fd, "r");
+			if (!data->fp) {
+				radlog(L_ERR, "FATAL: Failed to re-open detail file %s: %s",
+				       data->filename, strerror(errno));
+				exit(1);
+			}
+
 			/*
 			 *	Look for the header
 			 */
@@ -319,6 +389,15 @@ int detail_recv(rad_listen_t *listener,
 				goto open_file;
 			}
 
+			{
+				struct stat buf;
+				
+				fstat(listener->fd, &buf);
+				if (((off_t) ftell(data->fp)) == buf.st_size) {
+					goto cleanup;
+				}
+			}
+
 			/*
 			 *	End of file.  Delete it, and re-set
 			 *	everything.
@@ -326,7 +405,7 @@ int detail_recv(rad_listen_t *listener,
 			if (feof(data->fp)) {
 			cleanup:
 				unlink(data->filename_work);
-				fclose(data->fp); /* closes listener->fd */
+				if (data->fp) fclose(data->fp);
 				data->fp = NULL;
 				listener->fd = -1;
 				data->state = STATE_UNOPENED;
@@ -455,6 +534,14 @@ int detail_recv(rad_listen_t *listener,
 		 */
 		if (!strcasecmp(key, "Timestamp")) {
 			data->timestamp = atoi(value);
+
+			vp = paircreate(PW_PACKET_ORIGINAL_TIMESTAMP,
+					PW_TYPE_DATE);
+			if (vp) {
+				vp->vp_date = (uint32_t) data->timestamp;
+				*tail = vp;
+				tail = &(vp->next);
+			}
 			continue;
 		}
 
@@ -501,8 +588,8 @@ int detail_recv(rad_listen_t *listener,
 	 */
 	packet = rad_alloc(1);
 	if (!packet) {
-		data->state = STATE_NO_REPLY;	/* try again later */
-		return 0;	/* maybe memory will magically free up... */
+		radlog(L_ERR, "FATAL: Failed allocating memory for detail");
+		exit(1);
 	}
 
 	memset(packet, 0, sizeof(*packet));
@@ -588,11 +675,9 @@ int detail_recv(rad_listen_t *listener,
 	*pfun = rad_accounting;
 
 	if (debug_flag) {
-		printf("detail_recv: Read packet from %s\n", data->filename);
+		fr_printf_log("detail_recv: Read packet from %s\n", data->filename_work);
 		for (vp = packet->vps; vp; vp = vp->next) {
-			putchar('\t');
-			vp_print(stdout, vp);
-			putchar('\n');
+			debug_pair(vp);
 		}
 	}
 
@@ -609,20 +694,6 @@ int detail_recv(rad_listen_t *listener,
 		return 0;
 	}
 
-	{
-		struct stat buf;
-
-		fstat(listener->fd, &buf);
-		if (((off_t) ftell(data->fp)) == buf.st_size) {
-			unlink(data->filename_work);
-			fclose(data->fp); /* closes listener->fd */
-			data->fp = NULL;
-			listener->fd = -1;
-			data->state = STATE_RUNNING;
-			return 1;
-		}
-	}
-
 	data->state = STATE_RUNNING;
 
 	return 1;
@@ -690,6 +761,7 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 	int		rcode;
 	listen_detail_t *data;
 	RADCLIENT	*client;
+	char buffer[2048];
 
 	if (!this->data) {
 		this->data = rad_malloc(sizeof(*data));
@@ -697,6 +769,7 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 	}
 
 	data = this->data;
+	data->delay_time = USEC;
 
 	rcode = cf_section_parse(cs, data, detail_config);
 	if (rcode < 0) {
@@ -714,6 +787,34 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 		return -1;
 	}
 
+	/*
+	 *	If the filename is a glob, use "detail.work" as the
+	 *	work file name.
+	 */
+	if ((strchr(data->filename, '*') != NULL) ||
+	    (strchr(data->filename, '[') != NULL)) {
+		char *p;
+
+#ifndef HAVE_GLOB_H
+		radlog(L_INFO, "WARNING: Detail file \"%s\" appears to use file globbing, but it is not supported on this system.", data->filename);
+#endif
+		strlcpy(buffer, data->filename, sizeof(buffer));
+		p = strrchr(buffer, FR_DIR_SEP);
+		if (p) {
+			p[1] = '\0';
+		} else {
+			buffer[0] = '\0';
+		}
+		strlcat(buffer, "detail.work",
+			sizeof(buffer) - strlen(buffer));
+			
+	} else {
+		snprintf(buffer, sizeof(buffer), "%s.work", data->filename);
+	}
+
+	free(data->filename_work);
+	data->filename_work = strdup(buffer); /* FIXME: leaked */
+
 	data->vps = NULL;
 	data->fp = NULL;
 	data->state = STATE_UNOPENED;
@@ -732,3 +833,4 @@ int detail_parse(CONF_SECTION *cs, rad_listen_t *this)
 
 	return 0;
 }
+#endif
