@@ -1,7 +1,7 @@
 /*
  * listen.c	Handle socket stuff
  *
- * Version:	$Id: listen.c,v 1.115 2008/01/01 17:29:12 aland Exp $
+ * Version:	$Id: listen.c,v 1.125 2008/06/06 12:56:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,13 +22,14 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: listen.c,v 1.115 2008/01/01 17:29:12 aland Exp $")
+RCSID("$Id: listen.c,v 1.125 2008/06/06 12:56:22 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/radius_snmp.h>
 #include <freeradius-devel/modules.h>
 #include <freeradius-devel/rad_assert.h>
 #include <freeradius-devel/vqp.h>
+#include <freeradius-devel/dhcp.h>
 
 #include <freeradius-devel/vmps.h>
 #include <freeradius-devel/detail.h>
@@ -84,7 +85,8 @@ RADCLIENT *client_listener_find(const rad_listen_t *listener,
 
 	rad_assert((listener->type == RAD_LISTEN_AUTH) ||
 		   (listener->type == RAD_LISTEN_ACCT) ||
-		   (listener->type == RAD_LISTEN_VQP));
+		   (listener->type == RAD_LISTEN_VQP) ||
+		   (listener->type == RAD_LISTEN_DHCP));
 
 	clients = ((listen_socket_t *)listener->data)->clients;
 
@@ -136,6 +138,7 @@ static int rad_status_server(REQUEST *request)
 		}
 		break;
 
+#ifdef WITH_ACCOUNTING
 	case RAD_LISTEN_ACCT:
 		dval = dict_valbyname(PW_ACCT_TYPE, "Status-Server");
 		if (dval) {
@@ -155,6 +158,7 @@ static int rad_status_server(REQUEST *request)
 			break;
 		}
 		break;
+#endif
 
 	default:
 		return 0;
@@ -182,13 +186,17 @@ static int socket_print(rad_listen_t *this, char *buffer, size_t bufsize)
 		name = "authentication";
 		break;
 
+#ifdef WITH_ACCOUNTING
 	case RAD_LISTEN_ACCT:
 		name = "accounting";
 		break;
+#endif
 
+#ifdef WITH_PROXY
 	case RAD_LISTEN_PROXY:
 		name = "proxy";
 		break;
+#endif
 
 #ifdef WITH_VMPS
 	case RAD_LISTEN_VQP:
@@ -196,6 +204,12 @@ static int socket_print(rad_listen_t *this, char *buffer, size_t bufsize)
 		break;
 #endif
 
+#ifdef WITH_DHCP
+	case RAD_LISTEN_DHCP:
+		name = "dhcp";
+		break;
+#endif
+
 	default:
 		name = "??";
 		break;
@@ -251,6 +265,12 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 			      &listen_port, "0");
 	if (rcode < 0) return -1;
 
+	if ((listen_port < 0) || (listen_port > 65500)) {
+			cf_log_err(cf_sectiontoitem(cs),
+				   "Invalid value for \"port\"");
+			return -1;
+	}
+
 	sock->ipaddr = ipaddr;
 	sock->port = listen_port;
 
@@ -299,12 +319,20 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 		} /* else it worked. */
 #endif
 	}
+
+#ifdef WITH_PROXY
+	/*
+	 *	Proxy sockets don't have clients.
+	 */
+	if (this->type == RAD_LISTEN_PROXY) return 0;
+#endif
 	
 	/*
 	 *	The more specific configurations are preferred to more
 	 *	generic ones.
 	 */
 	client_cs = NULL;
+	parentcs = cf_top_section(cs);
 	rcode = cf_item_parse(cs, "clients", PW_TYPE_STRING_PTR,
 			      &section_name, NULL);
 	if (rcode < 0) return -1; /* bad string */
@@ -312,7 +340,12 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 		/*
 		 *	Explicit list given: use it.
 		 */
-		client_cs = cf_section_find(section_name);
+		client_cs = cf_section_sub_find_name2(parentcs,
+						      "clients",
+						      section_name);
+		if (!client_cs) {
+			client_cs = cf_section_find(section_name);
+		}
 		if (!client_cs) {
 			cf_log_err(cf_sectiontoitem(cs),
 				   "Failed to find clients %s {...}",
@@ -323,7 +356,6 @@ static int common_socket_parse(CONF_SECTION *cs, rad_listen_t *this)
 		free(section_name);
 	} /* else there was no "clients = " entry. */
 
-	parentcs = cf_top_section(cs);
 	if (!client_cs) {
 		CONF_SECTION *server_cs;
 
@@ -368,6 +400,7 @@ static int auth_socket_send(rad_listen_t *listener, REQUEST *request)
 }
 
 
+#ifdef WITH_ACCOUNTING
 /*
  *	Send an accounting response packet (or not)
  */
@@ -387,8 +420,9 @@ static int acct_socket_send(rad_listen_t *listener, REQUEST *request)
 	return rad_send(request->reply, request->packet,
 			request->client->secret);
 }
+#endif
 
-
+#ifdef WITH_PROXY
 /*
  *	Send a packet to a home server.
  *
@@ -407,6 +441,7 @@ static int proxy_socket_send(rad_listen_t *listener, REQUEST *request)
 	return rad_send(request->proxy, request->packet,
 			request->home_server->secret);
 }
+#endif
 
 
 /*
@@ -513,6 +548,7 @@ static int auth_socket_recv(rad_listen_t *listener,
 }
 
 
+#ifdef WITH_ACCOUNTING
 /*
  *	Receive packets from an accounting socket
  */
@@ -615,8 +651,9 @@ static int acct_socket_recv(rad_listen_t *listener,
 	*pfun = fun;
 	return 1;
 }
+#endif
 
-
+#ifdef WITH_PROXY
 /*
  *	Recieve packets from a proxy socket.
  */
@@ -644,9 +681,11 @@ static int proxy_socket_recv(rad_listen_t *listener,
 		fun = rad_authenticate;
 		break;
 
+#ifdef WITH_ACCOUNTING
 	case PW_ACCOUNTING_RESPONSE:
 		fun = rad_accounting;
 		break;
+#endif
 
 	default:
 		/*
@@ -672,6 +711,7 @@ static int proxy_socket_recv(rad_listen_t *listener,
 
 	return 1;
 }
+#endif
 
 
 static int client_socket_encode(UNUSED rad_listen_t *listener, REQUEST *request)
@@ -698,6 +738,7 @@ static int client_socket_decode(UNUSED rad_listen_t *listener, REQUEST *request)
 			  request->client->secret);
 }
 
+#ifdef WITH_PROXY
 static int proxy_socket_encode(UNUSED rad_listen_t *listener, REQUEST *request)
 {
 	rad_encode(request->proxy, NULL, request->home_server->secret);
@@ -717,7 +758,7 @@ static int proxy_socket_decode(UNUSED rad_listen_t *listener, REQUEST *request)
 	return rad_decode(request->proxy_reply, request->proxy,
 			   request->home_server->secret);
 }
-
+#endif
 
 #ifdef WITH_SNMP
 static int radius_snmp_recv(rad_listen_t *listener,
@@ -754,28 +795,42 @@ static int radius_snmp_print(UNUSED rad_listen_t *this, char *buffer, size_t buf
 
 #endif
 
+#include "dhcpd.c"
+
 static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
 	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},	/* RAD_LISTEN_NONE */
 
+#ifdef WITH_PROXY
 	/* proxying */
-	{ NULL, NULL,
+	{ common_socket_parse, NULL,
 	  proxy_socket_recv, proxy_socket_send,
 	  socket_print, proxy_socket_encode, proxy_socket_decode },
+#else
+	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
+#endif
 
 	/* authentication */
 	{ common_socket_parse, NULL,
 	  auth_socket_recv, auth_socket_send,
 	  socket_print, client_socket_encode, client_socket_decode },
 
+#ifdef WITH_ACCOUNTING
 	/* accounting */
 	{ common_socket_parse, NULL,
 	  acct_socket_recv, acct_socket_send,
 	  socket_print, client_socket_encode, client_socket_decode},
+#else
+	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
+#endif
 
+#ifdef WITH_DETAIL
 	/* detail */
 	{ detail_parse, detail_free,
 	  detail_recv, detail_send,
 	  detail_print, detail_encode, detail_decode },
+#else
+	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
+#endif
 
 #ifdef WITH_VMPS
 	/* vlan query protocol */
@@ -786,6 +841,15 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
 	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
 #endif
 
+#ifdef WITH_DHCP
+	/* dhcp query protocol */
+	{ dhcp_socket_parse, NULL,
+	  dhcp_socket_recv, dhcp_socket_send,
+	  socket_print, dhcp_socket_encode, dhcp_socket_decode },
+#else
+	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL},
+#endif
+
 	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL}	/* RAD_LISTEN_SNMP */
 };
 
@@ -796,7 +860,6 @@ static const rad_listen_master_t master_listen[RAD_LISTEN_MAX] = {
  */
 static int listen_bind(rad_listen_t *this)
 {
-	rad_listen_t	**last;
 	listen_socket_t *sock = this->data;
 
 	/*
@@ -816,6 +879,7 @@ static int listen_bind(rad_listen_t *this)
 			}
 			break;
 
+#ifdef WITH_ACCOUNTING
 		case RAD_LISTEN_ACCT:
 			svp = getservbyname ("radacct", "udp");
 			if (svp != NULL) {
@@ -824,6 +888,13 @@ static int listen_bind(rad_listen_t *this)
 				sock->port = PW_ACCT_UDP_PORT;
 			}
 			break;
+#endif
+
+#ifdef WITH_PROXY
+		case RAD_LISTEN_PROXY:
+			sock->port = 0;
+			break;
+#endif
 
 #ifdef WITH_VMPS
 		case RAD_LISTEN_VQP:
@@ -837,34 +908,6 @@ static int listen_bind(rad_listen_t *this)
 		}
 	}
 
-	/*
-	 *	Find it in the old list, AFTER updating the port.  If
-	 *	it's there, use that, rather than creating a new
-	 *	socket.  This allows HUP's to re-use the old sockets,
-	 *	which means that packets waiting in the socket queue
-	 *	don't get lost.
-	 */
-	for (last = &mainconfig.listen;
-	     *last != NULL;
-	     last = &((*last)->next)) {
-		listen_socket_t *other;
-
-		if (this->type != (*last)->type) continue;
-
-		if ((this->type == RAD_LISTEN_DETAIL) ||
-		    (this->type == RAD_LISTEN_SNMP)) continue;
-
-		other = (listen_socket_t *)((*last)->data);
-
-		if ((sock->port == other->port) &&
-		    (sock->ipaddr.af == other->ipaddr.af) &&
-		    (fr_ipaddr_cmp(&sock->ipaddr, &other->ipaddr) == 0)) {
-			this->fd = (*last)->fd;
-			(*last)->fd = -1;
-			return 0;
-		}
-	}
-
 	this->fd = fr_socket(&sock->ipaddr, sock->port);
 	if (this->fd < 0) {
 		radlog(L_ERR, "ERROR: Failed to open socket: %s",
@@ -884,23 +927,28 @@ static int listen_bind(rad_listen_t *this)
 		memset(&src, 0, sizeof_src);
 		if (getsockname(this->fd, (struct sockaddr *) &src,
 				&sizeof_src) < 0) {
+			radlog(L_ERR, "Failed getting socket name: %s",
+			       strerror(errno));
 			return -1;
 		}
 
 		if (src.ss_family == AF_INET) {
-			struct sockaddr_in	*s4;
+			struct sockaddr_in	s4;
 			
-			s4 = (struct sockaddr_in *)&src;
-			sock->ipaddr.ipaddr.ip4addr = s4->sin_addr;
+			memcpy(&s4, &src, sizeof(s4));
+			sock->ipaddr.ipaddr.ip4addr = s4.sin_addr;
+			sock->port = ntohs(s4.sin_port);
 			
 #ifdef HAVE_STRUCT_SOCKADDR_IN6
 		} else if (src.ss_family == AF_INET6) {
-			struct sockaddr_in6	*s6;
+			struct sockaddr_in6	s6;
 			
-			s6 = (struct sockaddr_in6 *)&src;
-			sock->ipaddr.ipaddr.ip6addr = s6->sin6_addr;
+			memcpy(&s6, &src, sizeof(s6));
+			sock->ipaddr.ipaddr.ip6addr = s6.sin6_addr;
+			sock->port = ntohs(s6.sin6_port);
 #endif
 		} else {
+			radlog(L_ERR, "Socket has unsupported address family");
 			return -1;
 		}
 	}
@@ -950,13 +998,16 @@ static rad_listen_t *listen_alloc(RAD_LISTEN_TYPE type)
 	case RAD_LISTEN_ACCT:
 	case RAD_LISTEN_PROXY:
 	case RAD_LISTEN_VQP:
+	case RAD_LISTEN_DHCP:
 		this->data = rad_malloc(sizeof(listen_socket_t));
 		memset(this->data, 0, sizeof(listen_socket_t));
 		break;
 
+#ifdef WITH_DETAIL
 	case RAD_LISTEN_DETAIL:
 		this->data = NULL;
 		break;
+#endif
 
 	default:
 		rad_assert("Unsupported option!" == NULL);
@@ -967,6 +1018,7 @@ static rad_listen_t *listen_alloc(RAD_LISTEN_TYPE type)
 }
 
 
+#ifdef WITH_PROXY
 /*
  *	Externally visible function for creating a new proxy LISTENER.
  *
@@ -1035,14 +1087,26 @@ rad_listen_t *proxy_new_listener()
 	listen_free(&this);
 	return NULL;
 }
-
+#endif
 
 static const FR_NAME_NUMBER listen_compare[] = {
 	{ "auth",	RAD_LISTEN_AUTH },
+#ifdef WITH_ACCOUNTING
 	{ "acct",	RAD_LISTEN_ACCT },
+#endif
+#ifdef WITH_DETAIL
 	{ "detail",	RAD_LISTEN_DETAIL },
+#endif
+#ifdef WITH_PROXY
+	{ "proxy",	RAD_LISTEN_PROXY },
+#endif
 #ifdef WITH_VMPS
 	{ "vmps",	RAD_LISTEN_VQP },
+#endif
+#ifdef WITH_DHCP
+	{ "dhcp",	RAD_LISTEN_DHCP },
+#else
+	{ "dhcp",	RAD_LISTEN_NONE },
 #endif
 	{ NULL, 0 },
 };
@@ -1126,6 +1190,9 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 	rad_listen_t	*this;
 	fr_ipaddr_t	server_ipaddr;
 	int		auth_port = 0;
+#ifdef WITH_PROXY
+	int		defined_proxy = 0;
+#endif
 
 	/*
 	 *	We shouldn't be called with a pre-existing list.
@@ -1213,6 +1280,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 		if (strcmp(progname, "vmpsd") == 0) goto do_proxy;
 #endif
 
+#ifdef WITH_ACCOUNTING
 		/*
 		 *	Open Accounting Socket.
 		 *
@@ -1253,7 +1321,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 
 		*last = this;
 		last = &(this->next);
-
+#endif
 	} else if (mainconfig.port > 0) { /* no bind address, but a port */
 		radlog(L_ERR, "The command-line says \"-p %d\", but there is no associated IP address to use",
 		       mainconfig.port);
@@ -1262,9 +1330,38 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 
 	/*
 	 *	They specified an IP on the command-line, ignore
-	 *	all listen sections.
+	 *	all listen sections except the one in '-n'.
 	 */
-	if (mainconfig.myip.af != AF_UNSPEC) goto do_proxy;
+	if (mainconfig.myip.af != AF_UNSPEC) {
+		CONF_SECTION *subcs;
+		const char *name2 = cf_section_name2(cs);
+
+		cs = cf_section_sub_find_name2(config, "server",
+					       mainconfig.name);
+		if (!cs) goto do_proxy;
+
+		/*
+		 *	Should really abstract this code...
+		 */
+		for (subcs = cf_subsection_find_next(cs, NULL, "listen");
+		     subcs != NULL;
+		     subcs = cf_subsection_find_next(cs, subcs, "listen")) {
+			this = listen_parse(subcs, name2);
+			if (!this) {
+				listen_free(head);
+				return -1;
+			}
+
+#ifdef WITH_PROXY
+			if (this->type == RAD_LISTEN_PROXY) defined_proxy = 1;
+#endif
+			
+			*last = this;
+			last = &(this->next);
+		} /* loop over "listen" directives in server <foo> */
+
+		goto do_proxy;
+	}
 
 	/*
 	 *	Walk through the "listen" sections, if they exist.
@@ -1278,6 +1375,10 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 			return -1;
 		}
 
+#ifdef WITH_PROXY
+		if (this->type == RAD_LISTEN_PROXY) defined_proxy = 1;
+#endif
+
 		*last = this;
 		last = &(this->next);
 	}
@@ -1302,6 +1403,14 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 				return -1;
 			}
 			
+#ifdef WITH_PROXY
+			if (this->type == RAD_LISTEN_PROXY) {
+				radlog(L_ERR, "Error: listen type \"proxy\" Cannot appear in a virtual server section");
+				listen_free(head);
+				return -1;
+			}
+#endif
+
 			*last = this;
 			last = &(this->next);
 		} /* loop over "listen" directives in virtual servers */
@@ -1312,6 +1421,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 	 *	Otherwise, don't do anything.
 	 */
  do_proxy:
+#ifdef WITH_PROXY
 	if (mainconfig.proxy_requests == TRUE) {
 		int		port = -1;
 		listen_socket_t *sock = NULL;
@@ -1322,32 +1432,7 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 		 */
 		if (!*head) return -1;
 
-		/*
-		 *	If we previously had proxy sockets, copy them
-		 *	to the new config.
-		 */
-		if (mainconfig.listen != NULL) {
-			rad_listen_t *old, *next, **tail;
-
-			tail = &mainconfig.listen;
-			for (old = mainconfig.listen;
-			     old != NULL;
-			     old = next) {
-				next = old->next;
-
-				if (old->type != RAD_LISTEN_PROXY) {
-					tail = &((*tail)->next);
-					continue;
-				}
-
-				*last = old;
-				*tail = next;
-				old->next = NULL;
-				last = &(old->next);
-			}
-
-			goto do_snmp;
-		}
+		if (defined_proxy) goto do_snmp;
 
 		/*
 		 *	Find the first authentication port,
@@ -1411,19 +1496,11 @@ int listen_init(CONF_SECTION *config, rad_listen_t **head)
 		}
 	}
 
- do_snmp:
+ do_snmp:			/* used only in proxy code. */
+#endif
+
 #ifdef WITH_SNMP
 	if (radius_snmp_init(config)) {
-		/*
-		 *      Forget about the old one.
-		 */
-		for (this = mainconfig.listen;
-		     this != NULL;
-		     this = this->next) {
-			if (this->type != RAD_LISTEN_SNMP) continue;
-			this->fd = -1;
-		}
-
 		this = rad_malloc(sizeof(*this));
 		memset(this, 0, sizeof(*this));
 
