@@ -1,7 +1,7 @@
 /*
  * mainconf.c	Handle the server's configuration.
  *
- * Version:	$Id: mainconfig.c,v 1.144 2008/01/08 16:43:07 aland Exp $
+ * Version:	$Id: mainconfig.c,v 1.152 2008/06/05 09:48:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: mainconfig.c,v 1.144 2008/01/08 16:43:07 aland Exp $")
+RCSID("$Id: mainconfig.c,v 1.152 2008/06/05 09:48:22 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -50,6 +50,14 @@ RCSID("$Id: mainconfig.c,v 1.144 2008/01/08 16:43:07 aland Exp $")
 #	include <syslog.h>
 #endif
 
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
 struct main_config_t mainconfig;
 
 /*
@@ -60,7 +68,9 @@ static uid_t server_uid;
 static gid_t server_gid;
 static const char *uid_name = NULL;
 static const char *gid_name = NULL;
+static const char *chroot_dir = NULL;
 static int allow_core_dumps = 0;
+static const char *radlog_dest = NULL;
 
 /*
  *	These are not used anywhere else..
@@ -143,14 +153,30 @@ static const CONF_PARSER security_config[] = {
 /*
  *	Logging configuration for the server.
  */
-static const CONF_PARSER log_config[] = {
+static const CONF_PARSER logdest_config[] = {
+	{ "destination",  PW_TYPE_STRING_PTR, 0, &radlog_dest, "files" },
 	{ "syslog_facility",  PW_TYPE_STRING_PTR, 0, &syslog_facility, Stringify(0) },
-	{ "stripped_names", PW_TYPE_BOOLEAN, 0, &log_stripped_names,"no" },
 
 	{ "file", PW_TYPE_STRING_PTR, -1, &mainconfig.log_file, "${logdir}/radius.log" },
+	{ NULL, -1, 0, NULL, NULL }
+};
+
+
+static const CONF_PARSER serverdest_config[] = {
+	{ "log", PW_TYPE_SUBSECTION, 0, NULL, (const void *) logdest_config },
+	{ "log_file", PW_TYPE_STRING_PTR, -1, &mainconfig.log_file, NULL },
+	{ "log_destination", PW_TYPE_STRING_PTR, -1, &radlog_dest, NULL },
+	{ NULL, -1, 0, NULL, NULL }
+};
+
+
+static const CONF_PARSER log_config_nodest[] = {
+	{ "stripped_names", PW_TYPE_BOOLEAN, 0, &log_stripped_names,"no" },
+
 	{ "auth", PW_TYPE_BOOLEAN, -1, &mainconfig.log_auth, "no" },
 	{ "auth_badpass", PW_TYPE_BOOLEAN, 0, &mainconfig.log_auth_badpass, "no" },
 	{ "auth_goodpass", PW_TYPE_BOOLEAN, 0, &mainconfig.log_auth_goodpass, "no" },
+
 	{ NULL, -1, 0, NULL, NULL }
 };
 
@@ -181,15 +207,14 @@ static const CONF_PARSER server_config[] = {
 	{ "allow_core_dumps", PW_TYPE_BOOLEAN, 0, &allow_core_dumps, "no" },
 
 	{ "pidfile", PW_TYPE_STRING_PTR, 0, &mainconfig.pid_file, "${run_dir}/radiusd.pid"},
-	{ "user", PW_TYPE_STRING_PTR, 0, &uid_name, NULL},
-	{ "group", PW_TYPE_STRING_PTR, 0, &gid_name, NULL},
 	{ "checkrad", PW_TYPE_STRING_PTR, 0, &mainconfig.checkrad, "${sbindir}/checkrad" },
 
 	{ "debug_level", PW_TYPE_INTEGER, 0, &mainconfig.debug_level, "0"},
 
+#ifdef WITH_PROXY
 	{ "proxy_requests", PW_TYPE_BOOLEAN, 0, &mainconfig.proxy_requests, "yes" },
-	{ "log", PW_TYPE_SUBSECTION, 0, NULL,  (const void *) log_config},
-	{ "security", PW_TYPE_SUBSECTION, 0, NULL, (const void *) security_config },
+#endif
+	{ "log", PW_TYPE_SUBSECTION, 0, NULL, (const void *) log_config_nodest },
 
 	/*
 	 *	People with old configs will have these.  They are listed
@@ -203,11 +228,10 @@ static const CONF_PARSER server_config[] = {
 	{ "log_auth_badpass", PW_TYPE_BOOLEAN, 0, &mainconfig.log_auth_badpass, NULL },
 	{ "log_auth_goodpass", PW_TYPE_BOOLEAN, 0, &mainconfig.log_auth_goodpass, NULL },
 	{ "log_stripped_names", PW_TYPE_BOOLEAN, 0, &log_stripped_names, NULL },
-	{ "log_file", PW_TYPE_STRING_PTR, -1, &mainconfig.log_file, NULL },
+
 	{ NULL, -1, 0, NULL, NULL }
 };
 
-
 #define MAX_ARGV (256)
 
 
@@ -369,79 +393,42 @@ static int r_mkdir(const char *part)
 	return(0);
 }
 
-/*
- *	Checks if the log directory is writeable by a particular user.
- */
-static int radlogdir_iswritable(const char *effectiveuser)
-{
-#ifdef HAVE_GETPWNAM
-	struct passwd *pwent;
-#endif
-
-	if (!radlog_dir || FR_DIR_IS_RELATIVE(radlog_dir))
-		return(0);
-
-	if (r_mkdir(radlog_dir) != 0)
-		return(1);
-
-	/* FIXME: do we have this function? */
-	if (strstr(radlog_dir, "radius") == NULL)
-		return(0);
-
-	/* we have a logdir that mentions 'radius', so it's probably
-	 * safe to chown the immediate directory to be owned by the normal
-	 * process owner. we gotta do it before we give up root.  -chad
-	 */
-
-	if (!effectiveuser) {
-		return 1;
-	}
-
-#ifdef HAVE_GETPWNAM
-	pwent = getpwnam(effectiveuser);
-
-	if (pwent == NULL) /* uh oh! */
-		return(1);
-
-	if (chown(radlog_dir, pwent->pw_uid, -1) != 0)
-		return(1);
-#endif
-
-	return(0);
-}
-
 
 /*
+ *  Do chroot, if requested.
+ *
  *  Switch UID and GID to what is specified in the config file
  */
-static int switch_users(void)
+static int switch_users(CONF_SECTION *cs)
 {
 	int did_setuid = FALSE;
+	CONF_PAIR *cp;
 
 #ifdef HAVE_SYS_RESOURCE_H
 	struct rlimit core_limits;
 #endif
 
+	/*
+	 *	Don't do chroot/setuid/setgid if we're in debugging
+	 *	as non-root.
+	 */
+	if (debug_flag && (getuid() != 0)) return 1;
+
 #ifdef HAVE_GRP_H
 	/*  Set GID.  */
-	if (gid_name != NULL) {
+	cp = cf_pair_find(cs, "group");
+	if (cp) gid_name = cf_pair_value(cp);
+	if (gid_name) {
 		struct group *gr;
 
+		DEBUG2("group = %s", gid_name);
 		gr = getgrnam(gid_name);
 		if (gr == NULL) {
-			if (errno == ENOMEM) {
-				radlog(L_ERR, "Cannot switch to Group %s: out of memory", gid_name);
-			} else {
-				radlog(L_ERR, "Cannot switch group; %s doesn't exist", gid_name);
-			}
+			fprintf(stderr, "%s: Cannot get ID for group %s: %s\n",
+				progname, gid_name, strerror(errno));
 			return 0;
 		}
 		server_gid = gr->gr_gid;
-		if (setgid(server_gid) < 0) {
-			radlog(L_ERR, "Failed setting Group to %s: %s",
-			       gid_name, strerror(errno));
-			return 0;
-		}
 	} else {
 		server_gid = getgid();
 	}
@@ -449,39 +436,103 @@ static int switch_users(void)
 
 #ifdef HAVE_PWD_H
 	/*  Set UID.  */
-	if (uid_name != NULL) {
+	cp = cf_pair_find(cs, "user");
+	if (cp) uid_name = cf_pair_value(cp);
+	if (uid_name) {
 		struct passwd *pw;
-
+		
+		DEBUG2("user = %s", uid_name);
 		pw = getpwnam(uid_name);
 		if (pw == NULL) {
-			if (errno == ENOMEM) {
-				radlog(L_ERR, "Cannot switch to User %s: out of memory", uid_name);
-			} else {
-				radlog(L_ERR, "Cannot switch user; %s doesn't exist", uid_name);
-			}
+			fprintf(stderr, "%s: Cannot get passwd entry for user %s: %s\n",
+				progname, uid_name, strerror(errno));
 			return 0;
 		}
 		server_uid = pw->pw_uid;
 #ifdef HAVE_INITGROUPS
 		if (initgroups(uid_name, server_gid) < 0) {
-			if (errno != EPERM) {
-				radlog(L_ERR, "Failed setting supplementary groups for User %s: %s", uid_name, strerror(errno));
-				return 0;
-			}
+			fprintf(stderr, "%s: Cannot initialize supplementary group list for user %s: %s\n",
+				progname, uid_name, strerror(errno));
+			return 0;
 		}
 #endif
-		if (setuid(server_uid) < 0) {
-			radlog(L_ERR, "Failed setting User to %s: %s", uid_name, strerror(errno));
+	} else {
+		server_uid = getuid();
+	}
+#endif
+
+	cp = cf_pair_find(cs, "chroot");
+	if (cp) chroot_dir = cf_pair_value(cp);
+	if (chroot_dir) {
+		DEBUG2("chroot = %s", chroot_dir);
+		if (chroot(chroot_dir) < 0) {
+			fprintf(stderr, "%s: Failed to perform chroot %s: %s",
+				progname, chroot_dir, strerror(errno));
 			return 0;
 		}
 
 		/*
-		 *	Now core dumps are disabled on most secure systems.
+		 *	Note that we leave chdir alone.  It may be
+		 *	OUTSIDE of the root.  This allows us to read
+		 *	the configuration from "-d ./etc/raddb", with
+		 *	the chroot as "./chroot/" for example.  After
+		 *	the server has been loaded, it does a "cd
+		 *	${logdir}" below, so that core files (if any)
+		 *	go to a logging directory.
+		 *
+		 *	This also allows the configuration of the
+		 *	server to be outside of the chroot.  If the
+		 *	server is statically linked, then the only
+		 *	things needed inside of the chroot are the
+		 *	logging directories.
 		 */
-		did_setuid = TRUE;
+		radlog(L_INFO, "performing chroot to %s\n", chroot_dir);
+	}
+
+#ifdef HAVE_GRP_H
+	/*  Set GID.  */
+	if (gid_name && (setgid(server_gid) < 0)) {
+		fprintf(stderr, "%s: Failed setting group to %s: %s",
+			progname, gid_name, strerror(errno));
+		return 0;
 	}
 #endif
 
+#ifdef HAVE_PWD_H
+	if (uid_name && (setuid(server_uid) < 0)) {
+		fprintf(stderr, "%s: Failed setting user to %s: %s",
+			progname, uid_name, strerror(errno));
+		return 0;
+	}
+
+	/*
+	 *	Now core dumps are disabled on most secure systems.
+	 */
+	did_setuid = TRUE;
+#endif
+
+	/*
+	 *	Double check that we can write to the log directory.
+	 *
+	 *	If we can't, don't start, as we can't log any errors!
+	 */
+	if ((mainconfig.radlog_dest == RADLOG_FILES) &&
+	    (mainconfig.log_file != NULL)) {
+		int fd = open(mainconfig.log_file,
+			      O_WRONLY | O_APPEND | O_CREAT, 0640);
+		if (fd < 0) {
+			fprintf(stderr, "%s: Cannot write to log file %s: %s\n",
+				progname, mainconfig.log_file, strerror(errno));
+			return 0;
+		}
+		close(fd);
+
+		/*
+		 *	After this it's safe to call radlog(), as it's going
+		 *	to the right place.
+		 */
+	}
+
 #ifdef HAVE_SYS_RESOURCE_H
 	/*  Get the current maximum for core files.  */
 	if (getrlimit(RLIMIT_CORE, &core_limits) < 0) {
@@ -552,18 +603,6 @@ static int switch_users(void)
 	 *	   core dump capabilities inherited from the parent shell.
 	 */
 
-#if defined(HAVE_PWD_H) && defined(HAVE_GRP_H)
-	/*
-	 *	We've probably written to the log file already as
-	 *	root.root, so if we have switched users, we've got to
-	 *	update the ownership of the file.
-	 */
-	if ((debug_flag == 0) &&
-	    (mainconfig.radlog_dest == RADLOG_FILES) &&
-	    (mainconfig.log_file != NULL)) {
-		chown(mainconfig.log_file, server_uid, server_gid);
-	}
-#endif
 	return 1;
 }
 
@@ -628,87 +667,39 @@ int read_mainconfig(int reload)
 		return -1;
 	}
 
-#if 0
 	/*
-	 *	Add templates to each kind of subsection.
+	 *	If there was no log destination set on the command line,
+	 *	set it now.
 	 */
-	templates = cf_section_sub_find(cs, "templates");
-	if (templates) {
-		CONF_SECTION *ts, *mycs;
-
-		/*
-		 *	Loop over the templates, adding them to the
-		 *	sections in the main configuration file.
-		 */
-		for (ts = cf_subsection_find_next(templates, NULL, NULL);
-		     ts != NULL;
-		     ts = cf_subsection_find_next(templates, ts, NULL)) {
-			const char *name1 = cf_section_name1(ts);
-
-			/*
-			 *	Loop over sections in the main config
-			 *	file, adding templats.
-			 */
-			for (mycs = cf_subsection_find_next(cs, NULL, name1);
-			     mycs != NULL;
-			     mycs = cf_subsection_find_next(cs, mycs, name1)) {
-				const char *value;
-
-				value = cf_section_value_find(mycs, "template");
-				if (value) {
-					CONF_SECTION *tts;
-
-					tts = cf_section_sub_find_name2(templates,
-									name1,
-									value);
-					if (!tts) {
-						radlog(L_ERR, "%s[%d]: Section refers to non-existent template \"%s\"",
-						       cf_section_filename(mycs), cf_section_lineno(mycs), value);
-						return -1;
-					}
-					cf_section_template(mycs, tts);
-				} else {
-					cf_section_template(mycs, ts);
-				}
-			}
-		}
-	}
-#endif
-
-	/*
-	 *	Debug flag 1 MAY go to files.
-	 *	Debug flag 2 ALWAYS goes to stdout
-	 *
-	 *	Parse the log_destination before printing anything else.
-	 *	All messages before this MUST be errors, which log.c
-	 *	will print to stderr, since log_file is NULL, too.
-	 */
-	if (debug_flag < 2) {
-		int rcode;
-		char *radlog_dest = NULL;
-
-		rcode = cf_item_parse(cs, "log_destination",
-				      PW_TYPE_STRING_PTR, &radlog_dest,
-				      "files");
-		if (rcode < 0) return -1;
-
-		mainconfig.radlog_dest = fr_str2int(str2dest, radlog_dest, RADLOG_NUM_DEST);
-		if (mainconfig.radlog_dest == RADLOG_NUM_DEST) {
-			fprintf(stderr, "radiusd: Error: Unknown log_destination %s\n",
-				radlog_dest);
-			free(radlog_dest);
+	if (mainconfig.radlog_dest == RADLOG_NULL) {
+		if (cf_section_parse(cs, NULL, serverdest_config) < 0) {
+			fprintf(stderr, "radiusd: Error: Failed to parse log{} section.\n");
 			cf_section_free(&cs);
 			return -1;
 		}
-
+		
+		if (!radlog_dest) {
+			fprintf(stderr, "radiusd: Error: No log destination specified.\n");
+			cf_section_free(&cs);
+			return -1;
+		}
+		
+		mainconfig.radlog_dest = fr_str2int(str2dest, radlog_dest,
+						    RADLOG_NUM_DEST);
+		if (mainconfig.radlog_dest == RADLOG_NUM_DEST) {
+			fprintf(stderr, "radiusd: Error: Unknown log_destination %s\n",
+				radlog_dest);
+			cf_section_free(&cs);
+			return -1;
+		}
+		
 		if (mainconfig.radlog_dest == RADLOG_SYSLOG) {
 			/*
 			 *	Make sure syslog_facility isn't NULL
 			 *	before using it
 			 */
 			if (!syslog_facility) {
-				fprintf(stderr, "radiusd: Error: Unknown syslog chosen but no facility spedified\n");
-				free(radlog_dest);
+				fprintf(stderr, "radiusd: Error: Syslog chosen but no facility was specified\n");
 				cf_section_free(&cs);
 				return -1;
 			}
@@ -716,19 +707,17 @@ int read_mainconfig(int reload)
 			if (mainconfig.syslog_facility < 0) {
 				fprintf(stderr, "radiusd: Error: Unknown syslog_facility %s\n",
 					syslog_facility);
-				free(radlog_dest);
-				free(syslog_facility);
 				cf_section_free(&cs);
 				return -1;
 			}
 		}
-
-		free(radlog_dest);
-	} else {
-		mainconfig.radlog_dest = RADLOG_STDOUT;
-		mainconfig.radlog_fd = STDOUT_FILENO;
 	}
 
+	/*
+	 *	We should really switch users earlier in the process.
+	 */
+	if (!switch_users(cs)) exit(1);
+
 	/* Initialize the dictionary */
 	cp = cf_pair_find(cs, "dictionary");
 	if (cp) p = cf_pair_value(cp);
@@ -746,24 +735,6 @@ int read_mainconfig(int reload)
 	 */
 	cf_section_parse(cs, NULL, server_config);
 
-#if 0
-	/*
-	 *	Merge the old with the new.
-	 */
-	if (reload) {
-		CONF_SECTION *newcs;
-
-		newcs = cf_section_sub_find(cs, "modules");
-		oldcs = cf_section_sub_find(mainconfig.config, "modules");
-		if (newcs && oldcs) {
-			if (!cf_section_migrate(newcs, oldcs)) {
-				radlog(L_ERR, "Fatal error migrating configuration data");
-				return -1;
-			}
-		}
-	}
-#endif
-
 	/*
 	 *	Free the old configuration items, and replace them
 	 *	with the new ones.
@@ -814,23 +785,6 @@ int read_mainconfig(int reload)
 	 *  changes.
 	 */
 
-	/*
-	 * 	The first time around, ensure that we can write to the
-	 *	log directory.
-	 */
-	if (!reload && !debug_flag) {
-		/*
-		 *	We need root to do mkdir() and chown(), so we
-		 *	do this before giving up root.
-		 */
-		radlogdir_iswritable(uid_name);
-	}
-
-	/*
-	 *	We should really switch users earlier in the process.
-	 */
-	if (!switch_users()) exit(1);
-
 	/*
 	 *	Sanity check the configuration for internal
 	 *	consistency.
@@ -846,6 +800,14 @@ int read_mainconfig(int reload)
 		return -1;
 	}
 
+	if (chroot_dir) {
+		if (chdir(radlog_dir) < 0) {
+			radlog(L_ERR, "Failed to 'chdir %s' after chroot: %s",
+			       radlog_dir, strerror(errno));
+			return -1;
+		}
+	}
+
 	return 0;
 }
 
