@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.156 2007/12/31 12:46:04 aland Exp $
+ * Version:	$Id: modules.c,v 1.162 2008/06/06 12:56:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: modules.c,v 1.156 2007/12/31 12:46:04 aland Exp $")
+RCSID("$Id: modules.c,v 1.162 2008/06/06 12:56:22 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -54,7 +54,6 @@ typedef struct section_type_value_t {
 	int		attr;
 } section_type_value_t;
 
-
 /*
  *	Ordered by component
  */
@@ -69,6 +68,64 @@ static const section_type_value_t section_type_value[RLM_COMPONENT_COUNT] = {
 	{ "post-auth",    "Post-Auth-Type",  PW_POST_AUTH_TYPE },
 };
 
+
+#ifdef WITHOUT_LIBLTDL
+typedef struct lt_dlmodule_t {
+  const char	*name;
+  void		*ref;
+} lt_dlmodule_t;
+
+/*
+ *	Define modules here.
+ */
+extern module_t rlm_pap;
+extern module_t rlm_chap;
+extern module_t rlm_eap;
+
+/*
+ *	EAP structures are defined elsewhere.
+ */
+typedef struct eap_type_t EAP_TYPE;
+
+/*
+ *	And so on for other EAP types.
+ */
+extern EAP_TYPE rlm_eap_md5;
+
+static const lt_dlmodule_t lt_dlmodules[] = {
+	{ "rlm_pap", &rlm_pap },
+	{ "rlm_chap", &rlm_chap },
+	{ "rlm_eap", &rlm_eap },
+	{ "rlm_eap_md5", &rlm_eap_md5 },
+	
+	/*
+	 *	Add other modules here.
+	 */
+		
+	{ NULL, NULL }
+};
+
+
+lt_dlhandle lt_dlopenext(const char *name)
+{
+	int i;
+
+	for (i = 0; lt_dlmodules[i].name != NULL; i++) {
+		if (strcmp(name, lt_dlmodules[i].name) == 0) {
+			return lt_dlmodules[i].ref;
+		}
+	}
+
+	return NULL;
+}
+
+void *lt_dlsym(lt_dlhandle handle, UNUSED const char *symbol)
+{
+	return handle;
+}
+#endif /* WITHOUT_LIBLTDL */
+
+
 static void indexed_modcallable_free(void *data)
 {
 	indexed_modcallable *c = data;
@@ -427,7 +484,7 @@ static indexed_modcallable *new_sublist(const char *server, int comp, int idx)
 	return c;
 }
 
-static int indexed_modcall(int comp, int idx, REQUEST *request)
+int indexed_modcall(int comp, int idx, REQUEST *request)
 {
 	int rcode;
 	indexed_modcallable *this;
@@ -455,7 +512,7 @@ static int indexed_modcall(int comp, int idx, REQUEST *request)
  *	block
  */
 static int load_subcomponent_section(modcallable *parent, CONF_SECTION *cs,
-				     const char *server, int comp)
+				     const char *server, int attr, int comp)
 {
 	indexed_modcallable *subcomp;
 	modcallable *ml;
@@ -489,7 +546,7 @@ static int load_subcomponent_section(modcallable *parent, CONF_SECTION *cs,
 	 *	automatically.  If it isn't found, it's a serious
 	 *	error.
 	 */
-	dval = dict_valbyname(section_type_value[comp].attr, name2);
+	dval = dict_valbyname(attr, name2);
 	if (!dval) {
 		cf_log_err(cf_sectiontoitem(cs),
 			   "%s %s Not previously configured",
@@ -580,7 +637,9 @@ static int load_component_section(CONF_SECTION *cs,
 			if (strcmp(name1,
 				   section_type_value[comp].typename) == 0) {
 				if (!load_subcomponent_section(NULL, scs,
-							       server, comp)) {
+							       server,
+							       dattr->attr,
+							       comp)) {
 					return -1; /* FIXME: memleak? */
 				}
 				continue;
@@ -782,6 +841,38 @@ static int load_byserver(CONF_SECTION *cs)
 			}
 			flag = 1;
 		}
+
+#ifdef WITH_DHCP
+		if (!flag) {
+			const DICT_ATTR *dattr;
+
+			dattr = dict_attrbyname("DHCP-Message-Type");
+			if (!dattr) {
+				radlog(L_ERR, "No DHCP-Message-Type attribute");
+				return -1;
+			}
+
+			/*
+			 *	Handle each DHCP Message type separately.
+			 */
+			for (subcs = cf_subsection_find_next(cs, NULL,
+							     "dhcp");
+			     subcs != NULL;
+			     subcs = cf_subsection_find_next(cs, subcs,
+							     "dhcp")) {
+				const char *name2 = cf_section_name2(subcs);
+
+				DEBUG2(" Module: Checking dhcp %s {...} for more modules to load", name2);
+				if (!load_subcomponent_section(NULL, subcs,
+							       server,
+							       dattr->attr,
+							       RLM_COMPONENT_POST_AUTH)) {
+					return -1; /* FIXME: memleak? */
+				}
+				flag = 1;
+			}
+		}
+#endif
 	}
 
 	cf_log_info(cs, " }");
@@ -1001,10 +1092,11 @@ int setup_modules(int reload, CONF_SECTION *config)
 		     ci=cf_item_find_next(cs, ci)) {
 
 			/*
-			 *	Skip sections.  They'll be handled
-			 *	later, if they're referenced at all...
+			 *	Skip sections and "other" stuff.
+			 *	Sections will be handled later, if
+			 *	they're referenced at all...
 			 */
-			if (cf_item_is_section(ci)) {
+			if (!cf_item_is_pair(ci)) {
 				continue;
 			}
 
@@ -1096,6 +1188,7 @@ int module_authenticate(int auth_type, REQUEST *request)
 	return indexed_modcall(RLM_COMPONENT_AUTH, auth_type, request);
 }
 
+#ifdef WITH_ACCOUNTING
 /*
  *	Do pre-accounting for ALL configured sessions
  */
@@ -1111,7 +1204,9 @@ int module_accounting(int acct_type, REQUEST *request)
 {
 	return indexed_modcall(RLM_COMPONENT_ACCT, acct_type, request);
 }
+#endif
 
+#ifdef WITH_SESSION_MGMT
 /*
  *	See if a user is already logged in.
  *
@@ -1137,7 +1232,9 @@ int module_checksimul(int sess_type, REQUEST *request, int maxsimul)
 
 	return (request->simul_count < maxsimul) ? 0 : request->simul_mpp;
 }
+#endif
 
+#ifdef WITH_PROXY
 /*
  *	Do pre-proxying for ALL configured sessions
  */
@@ -1153,6 +1250,7 @@ int module_post_proxy(int type, REQUEST *request)
 {
 	return indexed_modcall(RLM_COMPONENT_POST_PROXY, type, request);
 }
+#endif
 
 /*
  *	Do post-authentication for ALL configured sessions
