@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.388 2008/01/10 09:06:56 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.393 2008/05/16 08:56:29 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: radiusd.c,v 1.388 2008/01/10 09:06:56 aland Exp $")
+RCSID("$Id: radiusd.c,v 1.393 2008/05/16 08:56:29 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/radius_snmp.h>
@@ -54,11 +54,6 @@ RCSID("$Id: radiusd.c,v 1.388 2008/01/10 09:06:56 aland Exp $")
 #	define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif
 
-#ifndef HAVE_PTHREAD_H
-#define thread_pool_lock(_x)
-#define thread_pool_unlock(_x)
-#endif
-
 /*
  *  Global variables.
  */
@@ -148,11 +143,12 @@ int main(int argc, char *argv[])
 	 *	Don't put output anywhere until we get told a little
 	 *	more.
 	 */
+	mainconfig.radlog_dest = RADLOG_NULL;
 	mainconfig.radlog_fd = -1;
 	mainconfig.log_file = NULL;
 
 	/*  Process the options.  */
-	while ((argval = getopt(argc, argv, "Cd:fhi:mn:p:svxX")) != EOF) {
+	while ((argval = getopt(argc, argv, "Cd:fhi:l:mn:p:stvxX")) != EOF) {
 
 		switch(argval) {
 			case 'C':
@@ -174,6 +170,11 @@ int main(int argc, char *argv[])
 				usage(0);
 				break;
 
+			case 'l':
+				mainconfig.log_file = strdup(optarg);
+				mainconfig.radlog_dest = RADLOG_FILES;
+				break;		  
+
 			case 'i':
 				if (ip_hton(optarg, AF_UNSPEC, &mainconfig.myip) < 0) {
 					fprintf(stderr, "radiusd: Invalid IP Address or hostname \"%s\"\n", optarg);
@@ -205,6 +206,10 @@ int main(int argc, char *argv[])
 				dont_fork = TRUE;
 				break;
 
+			case 't':	/* no child threads */
+				spawn_flag = FALSE;
+				break;
+
 			case 'v':
 				version();
 				break;
@@ -218,6 +223,7 @@ int main(int argc, char *argv[])
 				mainconfig.log_auth_goodpass = TRUE;
 				mainconfig.radlog_dest = RADLOG_STDOUT;
 				mainconfig.radlog_fd = STDOUT_FILENO;
+				fr_log_fp = stdout;
 				break;
 
 			case 'x':
@@ -241,7 +247,7 @@ int main(int argc, char *argv[])
 		radlog(L_INFO, "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n");
 		radlog(L_INFO, "PARTICULAR PURPOSE.\n");
 		radlog(L_INFO, "You may redistribute copies of FreeRADIUS under the terms of the\n");
-		radlog(L_INFO, "GNU General Public License.\n");
+		radlog(L_INFO, "GNU General Public License v2.\n");
 		fflush(NULL);
 	}
 
@@ -254,7 +260,7 @@ int main(int argc, char *argv[])
 	/*
 	 *  Disconnect from session
 	 */
-	if (debug_flag == 0 && dont_fork == FALSE) {
+	if (dont_fork == FALSE) {
 		pid_t pid = fork();
 
 		if (pid < 0) {
@@ -333,8 +339,7 @@ int main(int argc, char *argv[])
 	}
 
 	/*
-	 *	It's called the thread pool, but it does a little
-	 *	more than that.
+	 *	Initialize the event pool, including threads.
 	 */
 	radius_event_init(mainconfig.config, spawn_flag);
 
@@ -379,7 +384,7 @@ int main(int argc, char *argv[])
 	 *	Everything seems to have loaded OK, exit gracefully.
 	 */
 	if (check_config) {
-		DEBUG("Configuration appears OK.");
+		DEBUG("Configuration appears to be OK.");
 		exit(0);
 	}
 
@@ -387,10 +392,11 @@ int main(int argc, char *argv[])
 	 *	Process requests until HUP or exit.
 	 */
 	while ((rcode = radius_event_process()) == 0x80) {
+		radlog(L_INFO, "Received HUP.");
 		module_hup(cf_section_sub_find(mainconfig.config, "modules"));
 	}
 	
-	DEBUG("Exiting...");
+	radlog(L_INFO, "Exiting normally.");
 	
 	/*
 	 *	Ignore the TERM signal: we're
