@@ -1,7 +1,7 @@
 /*
  * session.c	session management
  *
- * Version:	$Id: session.c,v 1.43 2007/11/23 13:46:52 aland Exp $
+ * Version:	$Id: session.c,v 1.44 2008/06/05 10:18:53 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,15 +21,17 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: session.c,v 1.43 2007/11/23 13:46:52 aland Exp $")
+RCSID("$Id: session.c,v 1.44 2008/06/05 10:18:53 aland Exp $")
 
 #include	<freeradius-devel/radiusd.h>
+#include	<freeradius-devel/modules.h>
 #include	<freeradius-devel/rad_assert.h>
 
 #ifdef HAVE_SYS_WAIT_H
 #include	<sys/wait.h>
 #endif
 
+#ifdef WITH_SESSION_MGMT
 /*
  *	End a session by faking a Stop packet to all accounting modules.
  */
@@ -223,3 +225,22 @@ int rad_check_ts(UNUSED uint32_t nasaddr, UNUSED unsigned int portnum,
 	return 2;
 }
 #endif
+
+#else
+/* WITH_SESSION_MGMT */
+
+int session_zap(UNUSED REQUEST *request, UNUSED uint32_t nasaddr, UNUSED unsigned int port,
+		UNUSED const char *user,
+		UNUSED const char *sessionid, UNUSED uint32_t cliaddr, UNUSED char proto,
+		UNUSED int session_time)
+{
+	return RLM_MODULE_FAIL;
+}
+
+int rad_check_ts(UNUSED uint32_t nasaddr, UNUSED unsigned int portnum,
+		 UNUSED const char *user, UNUSED const char *session_id)
+{
+	radlog(L_ERR, "Simultaneous-Use is not supported");
+	return 2;
+}
+#endif
