@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.89 2007/10/22 08:40:47 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.91 2008/05/12 14:41:32 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: valuepair.c,v 1.89 2007/10/22 08:40:47 aland Exp $")
+RCSID("$Id: valuepair.c,v 1.91 2008/05/12 14:41:32 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -262,6 +262,23 @@ int radius_callback_compare(REQUEST *req, VALUE_PAIR *request,
 }
 
 
+/*
+ *	Find a comparison function for two attributes.
+ */
+int radius_find_compare(int attribute)
+{
+	struct cmp *c;
+
+	for (c = cmp; c; c = c->next) {
+		if (c->attribute == attribute) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
 /*
  *	See what attribute we want to compare with.
  */
@@ -700,3 +717,23 @@ VALUE_PAIR *radius_pairmake(REQUEST *request, VALUE_PAIR **vps,
 	return vp;
 }
 
+void debug_pair(VALUE_PAIR *vp)
+{
+	if (!vp || !debug_flag || !fr_log_fp) return;
+
+	fputc('\t', fr_log_fp);
+	vp_print(fr_log_fp, vp);
+	fputc('\n', fr_log_fp);
+}
+
+void debug_pair_list(VALUE_PAIR *vp)
+{
+	if (!vp || !debug_flag || !fr_log_fp) return;
+
+	while (vp) {
+		fputc('\t', fr_log_fp);
+		vp_print(fr_log_fp, vp);
+		fputc('\n', fr_log_fp);
+		vp = vp->next;
+	}
+}
