@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.179 2008/04/29 08:10:37 aland Exp $
+ * Version:	$Id: rlm_sql.c,v 1.181 2008/05/09 15:29:48 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: rlm_sql.c,v 1.179 2008/04/29 08:10:37 aland Exp $")
+RCSID("$Id: rlm_sql.c,v 1.181 2008/05/09 15:29:48 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -70,7 +70,7 @@ static const CONF_PARSER module_config[] = {
 	{"authorize_check_query", PW_TYPE_STRING_PTR,
 	 offsetof(SQL_CONFIG,authorize_check_query), NULL, ""},
 	{"authorize_reply_query", PW_TYPE_STRING_PTR,
-	 offsetof(SQL_CONFIG,authorize_reply_query), NULL, ""},
+	 offsetof(SQL_CONFIG,authorize_reply_query), NULL, NULL},
 	{"authorize_group_check_query", PW_TYPE_STRING_PTR,
 	 offsetof(SQL_CONFIG,authorize_group_check_query), NULL, ""},
 	{"authorize_group_reply_query", PW_TYPE_STRING_PTR,
@@ -857,6 +857,7 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance)
 		rlm_sql_detach(inst);
 		return -1;
 	}
+
 	paircompare_register(PW_SQL_GROUP, PW_USER_NAME, sql_groupcmp, inst);
 
 	if (inst->config->do_clients){
@@ -942,6 +943,10 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 		if (paircompare(request, request->packet->vps, check_tmp, &request->reply->vps) == 0) {
 			found = 1;
 			DEBUG2("rlm_sql (%s): User found in radcheck table", inst->config->xlat_name);
+
+			if (inst->config->authorize_reply_query &&
+			    *inst->config->authorize_reply_query) {
+
 			/*
 			 *	Now get the reply pairs since the paircompare matched
 			 */
@@ -964,9 +969,11 @@ static int rlm_sql_authorize(void *instance, REQUEST * request)
 				pairfree(&reply_tmp);
 				return RLM_MODULE_FAIL;
 			}
+
 			if (!inst->config->read_groups)
 				dofallthrough = fallthrough(reply_tmp);
 			pairxlatmove(request, &request->reply->vps, &reply_tmp);
+			}
 			pairxlatmove(request, &request->config_items, &check_tmp);
 		}
 	}
